; The first stage loader must disable interrupts before transferring control.
;
; The payload MUST be contiguous in physical memory.
;
; IMPORTANT: all memory accesses use forced address modes because we do not
; know what DP is set when this starts.
;
; This file must be assembled as a *raw* binary.

        ORG     $AF1F
kloader:
        LDS     #$FE00          ; Stack location
kloader0:
        bsr getbyte             ; fetch block flag
		bne postamble			; brif postamble
		bsr getbyte             ; get msb of block size
		stb >blocksize			; save it
		bsr getbyte             ; get lsb of block size
		stb >blocksize+1		; save it
		ldy >blocksize			; fetch block size
		bsr getbyte             ; get msb of load address
		stb >blockaddr			; save msb of address
		bsr getbyte             ; get lsb of load address
		stb >blockaddr+1		; save lsb of address
		ldu >blockaddr			; get load address
kloader1	bsr getbyte			; get data byte
		stb ,u+                 ; save in memory
		leay -1,y               ; are we done block yet?
		bne kloader1			; brif not
		bra kloader0			; go look for another "amble"
postamble	bsr getbyte			; skip unused byte
		bsr getbyte             ; skip unused byte
		bsr getbyte             ; get msb of exec address
		stb >blockaddr			; save it
		bsr getbyte             ; get lsb of exec address
		stb >blockaddr+1		; save it
		jmp [blockaddr]			; transfer control to the kernel
; this routine fetches bytes from the payload.
;
; Because the payload is SLZ compressed, it needs several variables to keep
; track of its operation. Also, because this is *stream* based, it cannot
; simply refer back to the location of previously decompressed data for the
; look behind references. That means it must maintain a 4096 byte ring buffer
; so every byte returned will be written to the current offset in the ring
; buffer. The ring buffer is stored starting at address 0 in RAM.
;
; Set up this routine by putting the first memory block number into cblock
; and the current source address in the block (plus $2000) into caddr. Set
; lbwptr to $0000 and tagcnt to zero.
savebuf		stb [lbwptr]			; save in buffer
		ldd >lbwptr			; bump ptr mod $1000
		addd #1
		anda #$0f
        ADDA   #$B0
		std >lbwptr
		rts
fetchbyte	lda >cblock			; fetch payload block
		ldx >caddr			; fetch payload pointer
		sta >$ffa6			; set MMU
		ldb ,x+				; get byte
		cmpx #$E000			; end of block?
		blo fb0				; brif not
		ldx #$C000			; reset pointer to start of block
		inca				; bump mmu pointer
		sta >cblock
fb0		stx >caddr			; save new payload pointer
		rts
getbyte		pshs a,b,x			; save regs
		lda >len			; get copy length
		beq gbnc			; brif not copying
gbcp		lda [lbptr]			; fetch byte
		sta 1,s				; save in return spot
		ldd >lbptr			; get address
		addd #1				; bump pointer
		anda #$0f
        ADDA   #$B0
		std >lbptr			; save new buffer pointer
		dec >len			; reduce copy length
gbret		ldb 1,s				; add byte to ring buffer
		bsr savebuf
		tst 1,s				; set flags
		puls a,b,x,pc			; return the byte
gbnc		lda >tagcnt			; get tag count
		bne gbot			; brif don't need a new tag byte
		bsr fetchbyte			; get byte from source block
		stb >tag			; save tag byte
		lda #8				; set tag counter
		sta >tagcnt
gbot		lsl >tag			; test next flag
		dec >tagcnt			; decrease tag counter (does not affect carry)
		bcs gblb			; brif we have a look behind
		bsr fetchbyte			; get literal byte
		stb 1,s				; put in return spot
		bra gbret			; return byte
gblb		bsr fetchbyte			; fetch next byte (MSB)
		stb ,--s			; save on stack
		andb #$0f			; calc length
		addb #2
		stb >len			; save length
		ldb ,s				; get back MSB
		lsrb				; shift offset bits over
		lsrb
		lsrb
		lsrb
		stb ,s				; save offset msb
		bsr fetchbyte			; get offset lsb
		stb 1,s				; save in offset value
		ldd >lbwptr			; get current ring buffer pointer
		subd ,s++			; subtract offset
		anda #$0f			; do mod 4096 on it
        ADDA   #$B0
		std >lbptr			; save source pointer
		bra gbcp			; return a byte
; temporary variables
blocksize	fdb 0				; size of block/kernel flag
blockaddr	fdb 0				; address of block/execution addr
cblock		fcb 0				; current block for reading bytes from
caddr		fdb payload-kloader+$C000     ; current address for reading bytes from + $2000
tag         fcb 0				; current tag byte (slz)
len         fcb 0				; current look behind copy length (slz)
tagcnt		fcb 0				; current tag counter (slz)
lbptr		fdb $0000			; current address reading in ring buffer (slz)
lbwptr		fdb $0000			; current address writing in ring buffer (slz)
buffer      EQU $B000			; reserve space for ring buffer
; the payload starts here
payload		equ *
