***********************************************************
* Sample audio data needs to be saved as PCM unsigned 8 Bit @ 6kHz
* V0.05 - Now decode temporarily to Mem BLK 00,01,02 - this is the main Video RAM location but it can be used here during initialization of the Audio Samples
*           Also now swaps bytes at the XX00 location with the xx00+$100 so that playback routines send the correct bytes to the DAC
***********************************************************
* This code must be used with the interrupts disabled ORCC #$50 and in memory $E000-$FD00 it temporarily uses mem banks 0-6 for decoding the audio samples
*        ORG     $E000
Setup_Audio:
        LDA     #$08                    * Copy MMU Bank 0 BLKs to MMU Bank 1
        LDX     #MMU_Reg_Bank0_0
!       LDB     ,X+
        STB     $7,X
        DECA
        BNE     <
*************************************
* Configure Audio settings
        LDA     PIA0_Byte_1_HSYNC		* SELECT SOUND OUT
        ANDA    #$F7                	* RESET LSB OF MUX BIT
        STA     PIA0_Byte_1_HSYNC		* STORE
        LDA     PIA0_Byte_3_VSYNC		* SELECT SOUND OUT
        ANDA    #$F7                	* RESET MSB OF MUX BIT
        STA     PIA0_Byte_3_VSYNC     	* STORE
        LDA     PIA1_Byte_3_IRQ_Ct_Snd	* GET PIA
        ORA     #$8                 	* SET 6-BIT SOUND ENABLE
        STA     PIA1_Byte_3_IRQ_Ct_Snd	* STORE

* This code masks off the two low bits written to $FF20 - we wont need this since we had to compress the audio and remove those bits anyway but it is a neat feature
* So you can send the PCM Unsigned 8 Bit sample as is, no masking needed
        LDA     PIA1_Byte_1_IRQ
        PSHS    A
        ANDA    #%00110011          	* FORCE BIT2 LOW
        STA     PIA1_Byte_1_IRQ       	* $FF20 NOW DATA DIRECTION REGISTER
        LDA     #%11111100          	* OUTPUT ON DAC, INPUT ON RS-232 & CDI
        STA     PIA1_Byte_0_IRQ
        PULS    A
        STA     PIA1_Byte_1_IRQ
*************************************
* Decompress the Audio samples and place them in the correct Mem Blocks

*************************
* A = source Mem BLK start location must be continuous Mem BLKS
* B = destination Mem BLK where the audio sample data will end up uncompressed
* X = audio sample start location in RAM of 6bit .shift data.  ie Snd_01_sound6bit.shift
* U = audio Sample End location in RAM of 6bit .shift data
* Returns X = Length of the Audio data
*************************
        LDD     #Audio08*256+Audio00	        * #$1A08
        LDX     #Snd_00_Cutscenes
        LDU     #Snd_00_Cutscenes_End
        JSR     PrepAudioData
        STX     Snd_00_Cutscenes_Length
		
        LDD     #Audio08*256+Audio01            * #$1A0C
        LDX     #Snd_01_Eat_Fruit
        LDU     #Snd_01_Eat_Fruit_End
        JSR     PrepAudioData
        STX     Snd_01_Eat_Fruit_Length
		
        LDD     #Audio08*256+Audio02            * #$1A0D
        LDX     #Snd_02_Eat_Ghost
        LDU     #Snd_02_Eat_Ghost_End
        JSR     PrepAudioData
        STX     Snd_02_Eat_Ghost_Length
		
        LDD     #Audio08*256+Audio03            * #$1A0E
        LDX     #Snd_03_Eat_Pill_1
        LDU     #Snd_03_Eat_Pill_1_End
        JSR     PrepAudioData
        STX     Snd_03_Eat_Pill_1_Length
		
        LDD     #Audio08*256+Audio04            * #$1A0F
        LDX     #Snd_04_Eat_Pill_2
        LDU     #Snd_04_Eat_Pill_2_End
        JSR     PrepAudioData
        STX     Snd_04_Eat_Pill_2_Length
		
        LDD     #Audio08*256+Audio07            * #$1A13
        LDX     #Snd_07_Insert_Coin
        LDU     #Snd_07_Insert_Coin_End
        BSR     PrepAudioData
        STX     Snd_07_Insert_Coin_Length
		
        LDD     #Audio09*256+Audio05            * #$1E10
        LDX     #Snd_05_Free_Man
        LDU     #Snd_05_Free_Man_End
        BSR     PrepAudioData
        STX     Snd_05_Free_Man_Length
		
        LDD     #Audio09*256+Audio06            * #$1E12
        LDX     #Snd_06_Ghost_Going_Home
        LDU     #Snd_06_Ghost_Going_Home_End
        BSR     PrepAudioData
        STX     Snd_06_Ghost_Going_Home_Length
		
        LDD     #Audio09*256+Audio08            * #$1E1A
        LDX     #Snd_08_Intro_Music
        LDU     #Snd_08_Intro_Music_End
        BSR     PrepAudioData
        STX     Snd_08_Intro_Music_Length
		
        LDD     #Audio12*256+Audio09            * #$221E
        LDX     #Snd_09_Pacman_Death
        LDU     #Snd_09_Pacman_Death_End
        BSR     PrepAudioData
        STX     Snd_09_Pacman_Death_Length
		
        LDD     #Audio12*256+Audio10            * #$2220
        LDX     #Snd_10_Power_Pill_Eaten
        LDU     #Snd_10_Power_Pill_Eaten_End
        BSR     PrepAudioData
        STX     Snd_10_Power_Pill_Eaten_Length
		
        LDD     #Audio12*256+Audio11            * #$2221
        LDX     #Snd_11_Siren
        LDU     #Snd_11_Siren_End
        BSR     PrepAudioData
        STX     Snd_11_Siren_Length

        LDD     #Audio12*256+Audio12            * #$2222
        LDX     #Snd_12_Siren_Faster
        LDU     #Snd_12_Siren_Faster_End
        BSR     PrepAudioData
        STX     Snd_12_Siren_Faster_Length

* All done setting up the Samples in RAM
* Put the memory pages back to the way they were
        LDA     #$08                    * Copy MMU Bank 0 BLKs to MMU Bank 1
        LDX     #MMU_Reg_Bank1_0
!       LDB     ,X+
        STB     -$9,X
        DECA
        BNE     <
        LDA     #$20                * Set the MMU bank registers to
        STA     INIT1_Register1		* Bank task 0 - Which is the normal 64k bank
        RTS

*************************
* A = source Mem BLK start location must be continuous Mem BLKS
* B = destination Mem BLK where the audio sample data will end up uncompressed
* X = audio sample start location in RAM.  ie Snd_01_sound6bit.shift
* U = audio Sample End location in RAM.
* Returns X = Length of the Audio data
*************************
EndOfSample:
        FDB     $0000
PrepAudioData:
        STA     MMU_Reg_Bank0_0         * Start Block for 6 bit shifted data files merged together
        INCA
        STA     MMU_Reg_Bank0_1
        INCA
        STA     MMU_Reg_Bank0_2
        INCA
        STA     MMU_Reg_Bank0_3         * 0000-7FFF
        CLRA                            * Move to Temp Destination Blocks of 00,01,02 - Use the Video RAM location during initialization
        STA     MMU_Reg_Bank0_4         * Store it in $8000-$DFFF
        INCA
        STA     MMU_Reg_Bank0_5
        INCA
        STA     MMU_Reg_Bank0_6
        STU     EndOfSample
        LDY     #$8000
!       LDA     ,X+
        STA     ,Y+
        CMPX    EndOfSample
        BLS     <
***************************************************************
* Decompress data from 6bit compressed data to 8 bit - Called from above sections
* Point compressed sample data at $8000 to $DFFF
* will decompress to $0000 to $7FFF
* Input A = Source Block start location, B = Destination Block start location
* will return with Y = length of uncompressed sample
***************************************************************
DecompresssAudio:
        STB     MMU_Reg_Bank0_0         * Uncompressed audio sample will be decompressed here $0000-$7FFF
        INCB
        STB     MMU_Reg_Bank0_1
        INCB
        STB     MMU_Reg_Bank0_2
        INCB
        STB     MMU_Reg_Bank0_3
        CLRA
        STA     MMU_Reg_Bank0_4         * Source audio sample in 6 bit .shift format will be here  BLKS 00,01,02 stored in $8000-$DFFF
        INCA
        STA     MMU_Reg_Bank0_5
        INCA
        STA     MMU_Reg_Bank0_6
        LDX     #$8002                  * Start location of the 6bit shift data
        LDU     #$0001                  * destination of uncompressed audio sample data, add 1 because $8000 wont be used
DeCompAudioLoop:
        LDA     ,X
        ANDA    #$FC
        STA     ,U+     * A now has the first byte
        LDD     ,X++
        ROLB
        ROLA
        ROLB
        ROLA
        ROLB
        ROLA
        ROLB
        ROLA
        ASLA
        ASLA            * A now has the 2nd byte, B has good top 4 bits of byte 3
        STA     ,U+
        LSRB
        LSRB
        LSRB
        LSRB            * B now has the good data in the low 4 bits
        LDA     ,X+
        ASLA
        ROLB
        ASLA            * A now has the 4th byte
        ROLB
        ASLB
        ASLB            * B now has the 3rd byte
        STB     ,U+
        STA     ,U+
        CMPU    $8000
        BLO     DeCompAudioLoop * Loop through all the data until we've unshifted all of the sample

        LDD     #$0000      * Fix the bytes at xx00 = the value at xx00+$100
!       TFR     D,X
        LEAX    $100,X
        LDA     ,X
        LEAX    -$100,X
        STA     ,X
        TFR     X,D
        INCA
        CMPD    $8000
        BLS     <
        LDX     $8000           * X = the length of the uncompressed audio sample
        RTS
************************************************
* Load the audio samples that have already been reversed and shifted together/compressed
* the code above will decompress them and put them into their proper memory blocks for playback later
* using the FIRQ
        ORG     MMU_Reg_Bank0_0			* Page $0000-$1FFF  Block #08
        FCB     Audio08
        FCB     Audio08a
        FCB     Audio08b
        FCB     Audio08c
		ORG		$0000
Snd_00_Cutscenes:
        INCLUDEBIN ./Pacman_Samples/00_Cutscenes_6khz_Mono_6Bit_rev.shift
Snd_00_Cutscenes_End:
Snd_01_Eat_Fruit:
        INCLUDEBIN ./Pacman_Samples/01_Eat_Fruit_6khz_8bit_Mono_rev.shift
Snd_01_Eat_Fruit_End:
Snd_02_Eat_Ghost:
        INCLUDEBIN ./Pacman_Samples/02_Eat_Ghost_6khz_8bit_Mono_rev.shift
Snd_02_Eat_Ghost_End:
Snd_03_Eat_Pill_1:
        INCLUDEBIN ./Pacman_Samples/03_Eat_Pill_1_6khz_8bit_Mono_rev.shift
Snd_03_Eat_Pill_1_End:
Snd_04_Eat_Pill_2:
        INCLUDEBIN ./Pacman_Samples/04_Eat_Pill_2_6khz_8bit_Mono_rev.shift
Snd_04_Eat_Pill_2_End:
Snd_07_Insert_Coin:
        INCLUDEBIN ./Pacman_Samples/07_Insert_Coin_6khz_8bit_Mono_rev.shift
Snd_07_Insert_Coin_End:
        ORG     MMU_Reg_Bank0_0			* Page $0000-$1FFF  Block #08
        FCB     Audio09
        FCB     Audio09a
        FCB     Audio10
        FCB     Audio11
		ORG		$0000
Snd_05_Free_Man:
        INCLUDEBIN ./Pacman_Samples/05_Free_Man_6khz_8bit_Mono_rev.shift
Snd_05_Free_Man_End:
Snd_06_Ghost_Going_Home:
        INCLUDEBIN ./Pacman_Samples/06_Ghost_Going_Home_6khz_8bit_Mono_loop4_rev.shift
Snd_06_Ghost_Going_Home_End:
Snd_08_Intro_Music:
        INCLUDEBIN ./Pacman_Samples/08_Intro_Music_6khz_8bit_Mono_rev.shift
Snd_08_Intro_Music_End:
        ORG     MMU_Reg_Bank0_0			* Page $0000-$1FFF  Block #08
        FCB     Audio12
        FCB     Audio13
		ORG		$0000
Snd_09_Pacman_Death:
        INCLUDEBIN ./Pacman_Samples/09_Pacman_Death_6khz_8bit_Mono_rev.shift
Snd_09_Pacman_Death_End:
Snd_10_Power_Pill_Eaten:
        INCLUDEBIN ./Pacman_Samples/10_Power_Pill_Eaten_6khz_8bit_Mono_rev.shift
Snd_10_Power_Pill_Eaten_End:
Snd_11_Siren:
        INCLUDEBIN ./Pacman_Samples/11_Siren_6khz_8bit_Mono_Low_Volume_rev.shift
Snd_11_Siren_End:
Snd_12_Siren_Faster:
        INCLUDEBIN ./Pacman_Samples/12_Siren_Faster_6khz_8bit_Mono_Low_Volume_rev.shift
Snd_12_Siren_Faster_End:

