		ORG		$2000
* Code to display Pcaman Death
* This will be loaded into RAM in mem block 1 = $2000-3FFF and executed when needed then control will go back to the main code
* This routine will use regular bitmap sprites and the code will make sure the Pac Man death anim does not over write the pac man maze or pellets still on screen.
* I'm using code from Lee Patterson's website where Lee Patterson, Pere Serrat, Simon Jonassen and Hugo Dufort worked on optimizing code for sprite rendering
DrawPacDie:
        LDD     #$0000
        SUBD    SpriteBuffer+16+2*4     * X & Y location of Sprite
    * Improve this loading of X value
        LDX     #Screen_Start-8-3+128*17  * X points to the top left corner of the playfield (skip over the score and top of the maze)
		LSRA                        * two pixels per byte so divide the pixel location by two
        LEAX    A,X                 * Move to the proper X coordinate (memory location)
        LDA     #128                * 128 bytes per row, move us down in the Y direction
        MUL                         * B = Y coordinate * 128 gives us the memory location to show our Sprite
        LEAX    D,X                 * X now has the place in Video RAM to draw the Sprite
        LEAX    128,X               * Adjust pointer down 1 pixel so death animation displays over where Pac Man was before dying
        TFR     X,D                 * Calculate which two memory blocks to get for writing to the screen memory in the Real VideoRAM and store them in $A000-$DFFF
        BITA    #$10
        BNE     >                   * Go if value is not betwwen 0000-0FFF or 2000-2FFF or 4000-4FFF or 6000-6FFF - handle it as normal
        BITA    #$E0
        BEQ     >                   * Skip if it's the range 0000-0FFF as we wont have any in this section
        CLRB                        * otherwise move it a block ahead so the sprite can be erased above it in memory
        ANDA    #$E0
        RORA                        * Shift A to the right 5 times to get the block that will hold our sprite
        RORA
        RORA
        RORA
        RORA
        DECA
        LDB     VidBuffPointer      * Pointer to Buffer that is currently not being shown - will be changed at the end of the IRQ
        BEQ     PacDieA@
        ADDA    #VideoRam1_0
PacDieA@:
        STA     MMU_Reg_Bank0_5
        INCA
        STA     MMU_Reg_Bank0_6
        TFR     X,D
        ANDA    #$1F
        ADDD    #$C000+8
        BRA     PacDieC@
!       CLRB                        * Clear the Carry flag
        ANDA    #$E0
        RORA                        * Shift A to the right 5 times to get the block that will hold our character
        RORA
        RORA
        RORA
        RORA
        LDB     VidBuffPointer      * Pointer to Buffer that is currently not being shown - will be changed at the end of the IRQ
        BEQ     PacDieB@
        ADDA    #VideoRam1_0
PacDieB@:
        STA     MMU_Reg_Bank0_5           * 0 to 3
        INCA
        STA     MMU_Reg_Bank0_6
        TFR     X,D
        ANDA    #$1F
        ADDD    #$A000+8
PacDieC@:
        PSHS    D                       * Save D on the stack as it now has the address in video RAM of where Pac Man is
        LDA     GetPacbackground        * Check if we should backup the background behind pacman (only happens when we first enter this routine)
        BEQ     AlreadyGotBackground    * Only copy if GetPacbackground = 1
        LDA     #PacmanCompSprite       * Erase Pacman so it's not in the copy of the background data'
        STA     MMU_Reg_Bank0_0         * Setup Mem BLKs $24 to page 0 - Pacman compiled sprites
        LDD     #$0000
        SUBD    RedGhostNum+16+2*4      * X & Y location of Sprite
    * Improve this loading of X value
        TFR     D,X
        LDB     #54                     * Sprite Number 54 is a blank (RTS)
        JSR     DrawPacSprite

!       PULS    X                       * Get the address where pacman is from the stack
        PSHS    X                       * save it back on the stack for later
        LDY     #BehindPacMan
        LDB     #16
!       LDU     ,X++
        STU     ,Y++
        LDU     ,X++
        STU     ,Y++
        LDU     ,X++
        STU     ,Y++
        LDU     ,X
        STU     ,Y++
        LEAX    122,X
        DECB
        BNE     <
        DEC     GetPacbackground        * Clear the flag that is used to indicate if we copied the background behind pacman
        LDA     #FruitCutSceneCompSprite
        STA     MMU_Reg_Bank0_0         * Set memory Page Blk 0 to Fruit and Pac Die animation sprites
        PULS    D,PC                    * RTS we haven't cleared the background yet'
AlreadyGotBackground:
        LDU     #PacDieSprites
        LDB     SpriteBuffer+2*4    * Sprite Number
        ANDB    #$3F                * Strip off any rotation info
        SUBB    #$34                * subtract the actual sprite number so it points to the correct sprite.
        LDA     #128                * Each sprite is 128 bytes of data, 8 bytes wide * 16 rows
        MUL                         * D = 128 * B
        LEAX    D,U                 * X now points to the correct Sprite address

*****************************************************************************************************************
* This cool bit of code is from Lee Patterson's website http://www.8bitcoder.com/optimizesprite/
* Where Lee Patterson, Pere Serrat, Simon Jonassen, Hugo Dufort, Roger Taylor and John Strong, worked on optimizing code for sprite rendering
* Thanks guys!
*
* The version I'm using below isn't the fastest version they came up with but it's almost and it's short and sweet.
* The version below is based off the version Lee Patterson called - sprite6c.asm found here - http://8bitcoder.com/source/optimizesprite/sprite6c.asm
*
* X = Sprite address
* Y = Background address
* U = destination address

        LDY     #BehindPacMan
        PULS    U               * U now points to the address in Video RAM to draw the sprite and background data mixed together
        LDB     #16             * B = 16 rows of sprite data to copy on the screen
!       PSHS    B
        LEAX    8,X
        STX     ctrl+1          * Do a little self modifying code to change when the loop should end (8 bytes per row)
        LEAX    -8,X
loop1:
        lda     ,y+             ; get background byte
        ldb     ,x+             ; get sprite byte
        beq     short           ; if transparent use full background
seeN1:
        bitb    #$f0            ; test sprite left nibble
        beq     tstR1           ; if zero use background left nibble
        anda    #$0f            ; if not zero, clear background left nibble
tstR1:
        bitb    #$0f            ; test sprite right nibble
        beq     doMix           ; if zero use background right nibble
        anda    #$f0            ; if not zero, clear background right nibble
doMix:
        ora     -1,x            ; add backgrouond and sprite
short:
        sta     ,u+             ; update destination and its pointer
ctrl:
        cmpx    #0000           ; got to the end of sprite data? (this value gets set each time we enter the routine to do a row, self modifying code)
        bne     loop1           ; no, loopback
        LEAU    120,U           * Move U to the start of the next row in video ram
        PULS    B
        DECB
        BNE     <
        RTS

BehindPacMan:
        RMB     128
