        ORG     $E000	           	* Adress of the Table and Data to be loaded
DirectPage:
        SETDP   DirectPage/256
*****************************************
* Audio off - Just re-enable the FIRQ and return
FIRQ_Interrupt0:
        STA     FIRQ0Restore+1
        LDA     FIRQENR        			* Re enable the FIRQ
FIRQ0Restore:
        LDA     #$00
        RTI								* Return from the FIRQ
*****************************************
* Play Sample in the background continuously in mem $FD00-$8000 when it hits $8000 it will loop back to it's start location
Sample1Start:
        FDB     $0000                   * location where sample starts counting down from, used for looping
FIRQ_Interrupt1:
* Play the Siren sample in the background continuously in mem block $8000-$9FFF when it hits $7FFF it will loop back to it's start location
		STA		FIRQRestoreA+1          * Save A
        LDA     FIRQENR                 * Re enable the FIRQ
        LDA     #$21                    * Set the MMU bank registers to
        STA     INIT1_Register1         * Bank task 1 - alternate 64k bank is now the current one
LoadAudio1:
		LDA		$8050
		STA     PIA1_Byte_0_IRQ         * OUTPUT sound to DAC
		DEC		LoadAudio1+2            * Decrement the LSB of Sample 1 pointer
		BNE		>                       * check if we hit 00, if so decrement the MSB of the sample pointer
		DEC		LoadAudio1+1            * Decrement the MSB of Sample 1 pointer (sample data modified so when we get to this point of XX00 it points to the correct byte of data)
		BMI		>                       * if negative then we are good still over $7FFF otherwise (end of sample 1, reset pointer)
Hit8000:                                * Restore the pointer to the start location of the sample
		LDA		Sample1Start            * Point to the MSB of the sample Start location
		STA		LoadAudio1+1            * Store the MSB of Sample 1 pointer
		LDA		Sample1Start+1          * Point to the LSB of the sample Start location
		STA		LoadAudio1+2            * Store the LSB of Sample 1 pointer
        
!       LDA     #$20                    * Set the MMU bank registers to
        STA     INIT1_Register1         * Bank task 0 - Back to the normal 64k bank
FIRQRestoreA:
		LDA		#$00                    * STA at the start of the FIRQ stores A's value here and gets loaded before the RTI saves a cycle and a byte of RAM
        RTI                             * Return from the FIRQ
******************************************************
* Playback Sample 1 continuously, play Sample 2 once
* Siren will be at the normal location in mem block $FD00-$8000 when it hits $7FFF it will loop back to it's start location
* 2nd sound will be in mem block $7FFF-$0000 backwards so we can count down to $0000 then leave this FIRQ when the sample is finished playing
* and jump to the main FIRQ just playing Sample 1 routine
Sample2Start:
        FDB     $0000                   * location where sample starts counting down from, used for looping
FIRQ_Interrupt2:
		STA		FIRQRestore2A+1      	* Save A
        LDA     FIRQENR        			* Re enable the FIRQ
        LDA     #$21                	* Set the MMU bank registers to
        STA     INIT1_Register1			* Bank task 1 - alternate 64k bank is now the current one
LoadAudio2:
		LDA		$6000					* Load Siren Sound sample same address as the normal FIRQ
AddAudio:
		ADDA	$6000					* Add 2nd sample points to address ($0000-$1fff) counting downwards
		RORA							* Divide the combined samples by two
		STA     PIA1_Byte_0_IRQ			* OUTPUT sound to DAC
		DEC		LoadAudio2+2			* Decrement the LSB of Sample 1 pointer
		BNE		Decrement_Snd2			* check if we hit 00, if so decrement the MSB of the sample pointer
		DEC		LoadAudio2+1            * Decrement the MSB of Sample 1 pointer (sample data modified so when we get to this point of XX00 it points to the correct byte of data)
		BMI		Decrement_Snd2     	 	* if negative then we are good still over $7FFF skip ahead, if not (end of sample, reset pointer)
Hit8000_2:                              * Restore the pointer to the start location of the sample
		LDA		Sample1Start            * Point to the MSB of the sample Start location
		STA		LoadAudio2+1            * Store the MSB of Sample 1 pointer
		LDA		Sample1Start+1          * Point to the LSB of the sample Start location
		STA		LoadAudio2+2            * Store the LSB of Sample 1 pointer
Decrement_Snd2:
		DEC		AddAudio+2				* Decrement the LSB of the 2nd sample pointer
		BNE		>                       * If we LSB of 2nd Sample reached 00 then exit routine
		DEC		AddAudio+1				* Decrement the MSB of the 2nd sample pointer (sample data modified so when we get to this point of XX00 it points to the correct byte of data)
		BPL		>						* If we haven't reached $0000 then we are a positive number so exit routine, exit if we are now at $FFFF
        LDA     LoadAudio2+1            * copy the sample pointer position from this FIRQ
        STA     LoadAudio1+1            * to the normal FIRQ sample pointer
        LDA     LoadAudio2+2            * copy the sample pointer position from this FIRQ
        STA     LoadAudio1+2            * to the normal FIRQ sample pointer
        LDA     #FIRQ_Interrupt1-DirectPage         * Point FIRQ to playback only the one sample in the $DFFF-$8000 range (where our silence is)
        STA     FIRQ_Start_Address                  * Point to the FIRQ Jump Vector
		
!       LDA     #$20                	* Set the MMU bank registers to
        STA     INIT1_Register1			* Bank task 0 - Back to the normal 64k bank
FIRQRestore2A:
		LDA		#$00                	* STA at the start of the FIRQ stores A's value here and gets loaded before the RTI saves a cycle and a byte of RAM
        RTI								* Return from the FIRQ
***************************************************************
*
* Memory locations used as temporary register storage since the Z80 has more registers
* then the 6809 and sometimes the Z80 registers are split up.  For example
* the register HL is quite often used as H and then L (similar to 6809 A and B) and then combined
* and read as an address (which can't be done frome 6809 D - register).
* Should store them in direct page (DP) memory
H_Temp      FCB     $00
L_Temp      FCB     $00
D_Temp      FCB     $00
E_Temp      FCB     $00
B_Temp      FCB     $00
C_Temp      FCB     $00
IX_Temp     FDB     $0000
* Use the Temp variable below while in the IRQ otherwise we might destroy the contents of the regular variables above
Misc01      FCB     $00             ; Quick Temp location - used for DECB when B is used for something else
VidBuffPointer:
            FCB     $00
DrawPointer FDB     $0000
Scroll_Pointer:
        FDB     $0000               * Used for autoscrolling the screen
* Needed for Character rendering
* Include the character data/background graphics in CoCo3 HSCREEN 2 ready format (16 colours)
* They will be flipped in to page 0 (MMU_Reg_0) when needed from Page 34 (68000-69FFF)
CharTable:  	EQU $0000
PacmanX:
        RMB     1
PacmanY:
        RMB     1
Char_Temp:
        RMB     2
CPalRight:
        RMB     1
CPalLeft:
        RMB     1
CPalBoth:
        RMB     1
Char_TempI:
        RMB     2
CPalRightI:
        RMB     1
CPalLeftI:
        RMB     1
CPalBothI:
        RMB     1
count:
        RMB     1                       * counter used for drwaing rows of the Sprite
TempAddress:
        FDB     $0000                   * Temp storage
OldRedLocation:
        FDB     $0000                   * Store Red ghost's old location so we can see if there is movement that needs to be dealt with drawing the sprite
OldPinkLocation:
        FDB     $0000                   * Store Pink ghost's old location so we can see if there is movement that needs to be dealt with drawing the sprite
OldBlueLocation:
        FDB     $0000                   * Store Blue ghost's old location so we can see if there is movement that needs to be dealt with drawing the sprite
OldOrangeLocation:
        FDB     $0000                   * Store Orange ghost's old location so we can see if there is movement that needs to be dealt with drawing the sprite
FruitSPriteSTemp:
        FDB     $0000
CutSceneFlag:
        FCB     $00
CutSceneCounter:
        FDB     $0000                   * Counter that will increment while the cutscene plays, it is used to determine when to play the cutscene music a 2nd time
DrawPacmanCutScene:
        FDB     $01                     * If = 1 then draw Pacman during cut scene, IF =0 then don't
GetPacbackground:                       * Flag if we need to backup the background behind pacman when pacman dies
        FDB     $00
Joystick:                               * If Joystick = 1 then use joystick routine
        FCB     $01
Input_JoystickX:                        * Value of joystick 0 - X value (0,32,63)
        FCB     $00
Input_JoystickY:                        * Value of joystick 0 - Y value (0,32,63)
        FCB     $00
ButtonsBandC:                           * Temp byte for B and C button value
        FCB     $00
Free_Man_Playing:                       * Flag if Free man Sample is being played, if = 0 then it is not currently playing
        FCB     $00
; ===========================================================================
; memset
; rst 8 = memset
; Fill (HL) to (HL+B) with value in A
; Modifies B,HL
; ===========================================================================
; =============== S U B R O U T I N E =======================================
Memset:     
        STA     ,X+                                 ;0008 77          LD      (HL),A  		; write A to (HL)
;        LEAX    1,X                                ;0009 23          INC     HL   			; next HL
        DECB
        BNE    	Memset                              ;000A 10FC        DJNZ    Memset   		; Decrement B,jump if B != 0
RetOpcode:                                   		
        RTS                                         ;000C C9          RET               	; This return is called from many places
; End of function memset
; ---------------------------------------------------------------------------
PrepDiff:
        JMP     PrepareDifficulty                   ;000D C30E07      JP      PrepareDifficulty ; TODO - This is called in MAME runs,- who calls it?
; ===========================================================================
* this isn't used anymore, there were two JSR's to this routine it was faster to just use LDA   A,X instead of jumping here'
;rst 10 (dereference pointer to bytes)
;HL points to table base
;a contains index
;HL += A; A = (HL);
;Modifies: HL,A
; =============== S U B R O U T I N E =======================================
;LookupByteA:
;        ADD     A,L                                ;0010 85          ADD     A,L            ; Add A to HL (one byte,then carry)
;        TFR     A,L                                ;0011 6F          LD      L,A            ; store back into L
;        LDA     #$0                                ;0012 3E00        LD      A,0            ; prepare to propogate carry
;        ADC     A,H - Fix                          ;0014 8C          ADC     A,H            ; and do it
;        TFR     A,H                                ;0015 67          LD      H,A            ; HL = HL+A
;        LEAX    A,X
;        LDA     A,X                                  ;0016 7E          LD      A,(HL)         ; Lookup A=(HL)
;        RTS                                         ;0017 C9          RET
; End of function LookupByteA
;=======================
; Lookup Word (HL)
;rst 18
;Dereference pointer to word
;HL = base address of table
;b contains index
;HL = (HL+2B)
;=======================
; =============== S U B R O U T I N E =======================================
LookupWordHL:
;        TFR     B,A                                 ;0018 78          LD      A,B
        TFR     B,A
        ASLA
;        BSR     LookupByteA                        ;001A D7          RST     10h            ; A = value stored at (HL+A)
;        LEAX    A,X
;        LDA     ,X
        LDX     A,X         ;TFR     A,E             ;001B 5F          LD      E,A            ; Low byte of word to lookup
;        LEAX    1,X                                ;001C 23          INC     HL             ; Next byte
;        LDD     ,X                                 ;001D 56          LD      D,(HL)         ; high byte
;        EXG     Y,X                                ;001E EB          EX      DE,HL          ; store in HL
        RTS                                         ;001F C9          RET
; End of function LookupWordHL
;======================
;Call with rst 20h
;Jump Table
;Calls function pointed to by (stack) + 2A,
;return returns to address on stack
;Called with rst 20 followed by
;table of function address
; TODO - make sure all rst 20 followed with table
;======================
; =============== S U B R O U T I N E =======================================
CallFunction:     
        PULS    X                                   ;0020 E1          POP     HL  			; get table address
        ASLA                                        ;0021 87          ADD     A,A    		; a = 2*a - spacing out values for jumping to a function
;        LBSR    LookupByteA                        ;0022 D7          RST     10h     		; dereference byte A = (HL+A)
;        LEAX    A,X
;        TFR     A,E                                ;0023 5F          LD      E,A    		; E=low byte of address to jump to
;        LEAX    1,X                                ;0024 23          INC     HL
;        LDB     ,X                                 ;0025 56          LD      D,(HL)   		; D = high address of address to jump to
;        EXG     Y,X                                ;0026 EB          EX      DE,HL   		; Prepare to jump
        LDX     A,X
        JMP     ,X                                  ;0027 E9          JP      (HL)  		; jump away! return address on stack
; End of function CallFunction
;=======================
; rst 28,Data
; Write the task into the TaskList
; The task routine is the byte following the
; instruction,And the parameter is the
; byte after that
; TODO - get list of tasks
;=======================
; =============== S U B R O U T I N E =======================================
InsertTask:     
; FUNCTION CHUNK AT ROM:0051 SIZE 00000014 BYTES
        PULS	X                                   ;0028 E1          POP     HL                ; get value on stack (address to return to,Example holds : 11h,see location 8A5)
        LDD    	,X++                                ;0029 46          LD      B,(HL)            ; get routine number (todo - or is thie parameter?)
;        LEAX    1,X                                ;002A 23          INC     HL
;        LDA     ,X+                                ;002B 4E          LD      C,(HL)            ; get parameter (or is this routine #?)
;        LEAX    1,X                                ;002C 23          INC     HL
        PSHS	X                                   ;002D E5          PUSH    HL                ; save address pass command
;        LBRA    ContinueTask                       ;002E 1812        JR      ContinueTask      ; skip the other rst instruction spaces
ContinueTask:                                       
        LDX     TaskListEndPtr                      ;0042 2A804C      LD    HL,(TaskListEndPtr) ; get next space to place task
        STD 	,X++                                ;0045 70          LD      (HL),B            ; save routine
;        INC     L                                  ;0046 2C          INC     L
;        STA     ,X                                 ;0047 71          LD      (HL),C            ; save parameter
;        INC     L                                  ;0048 2C          INC     L                 ; next task location
        CMPX    #TaskListNorm+$40
        BNE    	noOverflow                         	;0049 2002        JR      NZ,noOverflow		; did task pointer overflow?
        LDX     #TaskListNorm                       ;004B 2EC0        LD      L,0C0h ; '+'      ; yes,start at the table top
noOverflow:                                         
        STX     TaskListEndPtr                      ;004D 22804C      LD    (TaskListEndPtr),HL ; save location to put next task
        RTS                                         ;0050 C9          RET                       ; and jump back to instruction following the rst 28,Dw that called this
; End of function InsertTask
;===========================================================================
; InsertIRQTask
; inserts a task into the IRQ task list if possible
; call with rst 30,Data
; where data is the three bytes to store in task list
; Tasks list 4C90-4CBF (runs in IRQ)
; 16 entries,3 bytes each
; byte 0 - scheduled time
; byte 1 - index for jump table
; byte 2 - parameter passed in B
;===========================================================================
InsertIRQTask:                    
        LDY     #TaskListIRQ                        ;0030 11904C      LD      DE,4C90h			; first IRQ task address
        LDB     #$10                                ;0033 0610        LD      B,10h             ; look at 16 tasks in IRQ list
;        LBRA    IRQTaskLoop                        ;0035 C35100      JP      IRQTaskLoop      	; skip space for other rst handlers
; ---------------------------------------------------------------------------
; START OF FUNCTION CHUNK FOR InsertTask
IRQTaskLoop:                                 		
        LDA     ,Y                                  ;0051 1A          LD      A,(DE)           	; Get task launch time          
;        TSTA                                       ;0052 A7          AND     A             	; is there a task in this slot?
        BEQ    	InsertIRQTask2                     	;0053 2806        JR      Z,InsertIRQTask2 	; no,jump to add task
;        INC     E                                  ;0055 1C          INC     E                 ; increment 3 bytes to next task
;        INC     E                                  ;0056 1C          INC     E
;        INC     E                                  ;0057 1C          INC     E
        LEAY    3,Y
        DECB
        BNE    	IRQTaskLoop                        	;0058 10F7        DJNZ    IRQTaskLoop    	; if not done,Continue with next task
        RTS                                         ;005A C9          RET                       ; no more tasks
; ---------------------------------------------------------------------------
InsertIRQTask2:
        PULS    X                                   ;005B E1          POP     HL				; Get the return address where the task to insert is      
;        LDB     #$3                                ;005C 0603        LD      B,3            	; copy 3 bytes into the IRQ task list
IRQTaskRunLoop:
        LDD     ,X++                                ;005E 7E          LD      A,(HL)			; get a byte
        STD     ,Y++                                ;005F 12          LD      (DE),A         	; store in task list
        LDA     ,X+                                 ;005E 7E          LD      A,(HL)			; get a byte
        STA     ,Y                                  ;005F 12          LD      (DE),A         	; store in task list
;        LEAX    1,X                                ;0060 23          INC     HL              	; next source byte
;        INC     E                                  ;0061 1C          INC     E               	; next dest byte
;        DECB
;        BNE    	IRQTaskRunLoop                  ;0062 10FA        DJNZ    IRQTaskRunLoop  	; continue for all 3 bytes
        JMP     ,X                                  ;0064 E9          JP      (HL)            	; jump to address following the data after the rst 30
; END OF FUNCTION CHUNK FOR InsertTask
*****************************************************************
;=========================================
; reading any byte 5000h-503F gives IN0 and coin slot info
; Bit0 - joystick up    if 0
; Bit1 - joystick left  if 0
; Bit2 - joystick right if 0
; Bit3 - joystick down  if 0
; Bit4 - rack advance if 0 (go to next level)
; Bit5 - Coin slot 1 trigger by going from 0 to 1
; Bit6 - Coin slot 2 trigger by going from 0 to 1
; Bit7 - Credit button (0=pressed)
;=========================================
IN0_5000:
        FCB        $00                  ;5000                DB 0                   write 1 enable,0 disable
*****************************************************************
;=======================
; Reading bytes 5040h-507Fh return IN1
; 2nd player joystick and start buttons
; Bit0 - joystick up    if 0
; Bit1 - joystick left  if 0
; Bit2 - joystick right if 0
; Bit3 - joystick down  if 0
; Bit4 - board test 0 = on
; Bit5 - player 1 start,0 = on
; Bit6 - player 2 start,0 = on
; Bit7 - cocktail mode (0=table,1 = upright)
;=========================================
;=======================
;=======================
; Writing 5040h-507Fh modifies 3
; voice sound programming
; Note voice 1 is 5 nibbles of data,
; other two voices are 4 nibbles
;=======================
IN1_5040:
        FCB        $FF                  ;5040
*****************************************************************
* Temp IRQ - used while the code is gettting initialized
IRQ_Interrupt0:
        LDA		IRQENR                  * IRQENR - Interrupt request enable register
        RTI
