*************************************************************
* Draws a 8 x 8 bit 16 colour charater on the screen
* A = Character ASCII Code
* U = Destination location in Video RAM (Pac man screen memory $4000-$43FF) - Screen is rotated left so we have to acount for that location and rotation
* of the characters
* Handling of Pac Man Video RAM
* We Have a screen 32x36 total in an 8 pixel x 8 pixel location Pac Man uses only 28x36 leaving the left two chracters blank and the right two characters blank
*
* CoCo Screen
* $0000------$007F
* |              |
* |              |
* |              |
* |              |
* |              |
* |              |
* |              |
* |              |
* |              |
* |              |
* |              |
* $8F80------$8FFF
*
* Pac Man screen
* $43DD------$43C2   Line 1
* $43FD------$43E2   Line 2
* 4             44
* 3             00
* A             64
* 0             00
* |              |
* |              |
* |              |
* |              |
* 4             44
* 3             00
* B             75
* F             FF
* $401D------$4002    Line 3
* $403D------$4022    Line 4
*
* Addresses that aren't shown on screen are:
* 4000,4001,401E,401F
* 4020,4021,403E,403F
* 43C0,43C1,43DE,43DF
* 43E0,43E1,43FE,43FF
*

DrawCharacter:
		PSHS    D,X,Y,U
		LDB		#CharCompSprite0	* Set memory Blk Characters - Can't LDD here since we want to use A below
        STB     MMU_Reg_Bank0_0  	* into Page 0 & 1 which is $0000-$3FFF
        INCB
        STB     MMU_Reg_Bank0_1
        TFR     A,B
        CLRA
        LSLB
        ROLA                        * D now points to the offest for the Character table
        LDY     #Char_Table
        LEAY    D,Y                 * Y now points to the characters sprite code routine address
        LDY     ,Y                  * Y now has the routine address that will print the character
        CMPU    #$403F              * Let's see if it's in main screen area first since most data will be here
        BHI     CheckMain
BotScreen:
        CMPU    #$4002
        LBLO    NoShow
        CMPU    #$401E
        BLO     Line3
        CMPU    #$4022
        LBLO    NoShow
        CMPU    #$403D
        LBHI    NoShow
Line4:
        STU     Char_Temp
        LDD     #$403D
        SUBD    Char_Temp
        ASLB
        ASLB                        	* B * 4 (4 bytes per character offeset for X value)
        LDU     #Screen_Start+128*8*35  * Bottom Line Starting Point
        LEAU    B,U                 	* Add the X value
        LBRA    ShowCharacter
Line3:
        STU     Char_Temp
        LDD     #$401D
        SUBD    Char_Temp
        ASLB
        ASLB                        	* B * 4 (4 bytes per character offeset for X value)
        LDU     #Screen_Start+128*8*34  * 2nd from bottom Line Starting Point
        LEAU    B,U                 	* Add the X value
        BRA     ShowCharacter
CheckMain:
        CMPU    #$43C0
        BLO     ShowMain
CheckTop:
        CMPU    #$43C2
        LBLO    NoShow
        CMPU    #$43DE
        BLO     Line1
        CMPU    #$43E2
        LBLO    NoShow
        CMPU    #$43FD
        LBHI    NoShow
Line2:
        STU     Char_Temp
        LDD     #$43FD
        SUBD    Char_Temp
        ASLB
        ASLB                        * B * 4 (4 bytes per character offeset for X value)
        LDU     #Screen_Start+128*8 * 2nd Line Starting Point
        LEAU    B,U                 * Add the X value
        BRA     ShowCharacter
Line1:
        STU     Char_Temp
        LDD     #$43DD
        SUBD    Char_Temp
        ASLB
        ASLB                        * B * 4 (4 bytes per character offeset for X value)
        LDU     #Screen_Start       * Top Line Starting Point
        LEAU    B,U                 * Add the X value
        BRA     ShowCharacter
ShowMain:
        TFR     U,D
        PSHS    B                   * Pacman Y will be 5 LS bits of B
        ANDB    #$1F
        STB     PacmanY
        PULS    B
        RORA
        RORB
        RORA
        RORB
        RORA
        RORB
        RORA
        RORB
        RORA
        RORB
        ANDB    #$1F                    * B-2 = X Value of Pacman Hardware where 0 is the right side and 27 is the left side
        STB     PacmanX
        LDA     #29
        SUBA    PacmanX                 * A now has proper X value for our character screen
        ASLA
        ASLA                            * A * 4 (4 bytes per character offeset for X value)
        LDU     #Screen_Start+128*16    * 2 top rows must be skipped for the proper X and Y character location to start from
        LEAU    A,U                     * Now it points to the correct X location
        LDB     PacmanY
        LDA     #8
        MUL                             * D now has our proper Y Character pointer location
        LDA     #128                    * 8 * 128 = 1024 * Y value is the real location for our screen
        MUL
        LEAU    D,U                     * Add it together and Now U points to the correct Screen location
ShowCharacter:
        TFR     U,D
        CLRB                            * Clear the Carry flag
        ANDA    #$E0
        RORA                            * Shift A to the right 5 times to get the block that will hold our character
        RORA
        RORA
        RORA
        RORA
        PSHS    A
        STA     MMU_Reg_Bank0_5  		* Setup block of real VideoRAM at $A000-$BFFF
        TFR     U,D
        ANDA    #$1F
        ADDD    #$A004
        TFR     D,X
        PSHS    X,Y
        JSR     ,Y
        PULS    X,Y
        PULS    A
;        PSHS    A,X,Y
;        STA     MMU_Reg_Bank0_5  		* Setup block of real VideoRAM at $A000-$BFFF
;        JSR     ,Y
;        PULS    A,X,Y
        ADDA    #VideoRam1_0
        STA     MMU_Reg_Bank0_5  		* Setup block of real VideoRAM at $A000-$BFFF
        JSR     ,Y
NoShow:                         		* Jump's here to skip showing any character as it is pointing to one of the special not shown locations
										* Set memory Page Blk 0 & 1 back to Stored value into Page 0 which is $0000-$1FFF
;		LDD		#RegRAM0*256+RegRAM1	* Set memory Page Blk 0 & 1 to Normal Pacman and Ghost Sprites
;        STD     MMU_Reg_Bank0_0  		* into Page 0 & 1 which is $0000-$3FFF
		PULS    D,X,Y,U,PC
