* V05 - changed to have 2 seperate routines depending if we are on Vid Buffer 0 or 1
*       it will take twice the memory but it will reduce all the checking during the routine
*     - Takes Sprite palette for input now and uses that info to figure out what type of sprite to draw
* V04 using Double buffering
*************************************************************
* DrawRedSprite:  Similar to DrawSpriteNew but we can erase whatever Red Ghost runs over, no need to restore dots or pills behind...
* Draws a 16 x 16 bit 16 colour sprite on the screen
* Compiled sprites routine blocks will already be in page 0 & 1
* X = X & Y coordinates
* A = Sprite source #
* B = Sprite Palette #   ($00 = Invisible, $11 = Dark Blue Ghost, $12 = White/Red Ghost, $18 = Score, $19 = Eyes going home)
DrawRedSprite:
        CMPB    #$01                * Normal Red Ghost Colour (most of the time)
        BEQ     RedNormal           * Jump ahead if so
        CMPB    #$11                * If Palette is $11 then Ghosts are Dark Blue
        BEQ     RedNormal           * Setup sprite as Dark Blue Ghost show same table # as Normal Ghosts
        CMPB    #$12                * If Palette is $12 then Ghosts are White/Red changing back to normal after Power Pill eaten
        BEQ     RedWhiteRed         * Setup sprite as White/Red Ghost
        CMPB    #$19                * If Palette is $19 then just ghost eyes showing
        BEQ     RedShowEyes         * Anything Else is the Score show same table # as Normal Ghosts
        CMPB    #$18                * If Palette is $18 then Score so then treat as normal
        BEQ     RedNormal           * Setup sprite as Score, show same table # as Normal Ghosts
* If we get this far then it is an invisible sprite so
RedInvisible:
        LDA     #24                 * Entry 25 in table is a skip or simply RTS (Do not draw anything)
        BRA     >
RedShowEyes:
		SUBA    #16                 * Adjust Sprite pointer to the Red Ghost Table position
        BRA     >
RedWhiteRed:
		SUBA    #26                 * Adjust Sprite pointer to the Red Ghost Table position
        BRA     >
RedNormal:
		SUBA    #28                 * Adjust Sprite pointer to the Red Ghost Table position
!       PSHS    A,X                 * Save the new X & Y coordinates and the new sprite # to be drawn
        LDA     VidBuffPointer      * Pointer to Buffer that is currently not being shown - will be changed at the end of the IRQ
        LBEQ    EraseRedBuff0       * If 0 go update Buff0
EraseRedBuff1:                      * First erase old Red Ghost
        LDX     OldRedLocation1
        LDD     OldRedMemBlk1       * A=OldRedMemBlk1, B=OldRedSprite1
        STA     MMU_Reg_Bank0_5     * Point to the old Mem Block where Red Ghost was last written
        INCA
        STA     MMU_Reg_Bank0_6
        LDU     #RedTableErase      * Point to the blank version of the Red Ghost sprite
        CLRA
        JSR     [D,U]               * draw the blank Red Ghost sprite
        LDA     OldRedDirection1
        LDU     OldRedLocation1
        LDY     OldRedXYPos1
        JSR     RestoreBehindGhosts
        PULS    B,X                 * restore the registers
        LSLB
        PSHS    B                   * Save pointer to sprite table on the stack
        TFR     X,D
        CMPA    #$09                * If X or Y co-ordinates are less then 9 then don't draw the sprite - Real Pacman hardware sprites would be off screen
        BLO     >
        CMPB    #$09
        BLO     >
        CMPA    #$E5                * If X or Y co-ordinates are above $E5 then don't draw the sprite - Real Pacman hardware sprites would be off screen
        BHI     >
        CMPB    #$EE
        BHI     >
        BRA     DrawNewRed1
!       PULS    B,PC
DrawNewRed1:
        LDX     #Screen_Start-8-3+128*17  * X points to the top left corner of the playfield (skip over the score and top of the maze)
		LSRA                        * two pixels per byte so divide the pixel location by two
		PSHS	CC					* if the carry is now set then X position is odd
        LEAX    A,X                 * Move to the proper X coordinate (memory location)
        LDA     #128                * 128 bytes per row, move us down in the Y direction
        MUL                         * B = Y coordinate * 128 gives us the memory location to show our Sprite
        LEAX    D,X                 * X now has the place in Video RAM to draw the Sprite
        TFR     X,D                 * Calculate which two memory blocks to get for writing to the screen memory in the Real VideoRAM and store them in $A000-$DFFF
        BITA    #$10
        BNE     >                   * Go if value is not betwwen 0000-0FFF or 2000-2FFF or 4000-4FFF or 6000-6FFF - handle it as normal
        BITA    #$E0
        BEQ     >                   * Skip if it's the range 0000-0FFF as we wont have any in this section
        CLRB                        * otherwise move it a block ahead so the sprite can be erased above it in memory
        ANDA    #$E0
        RORA                        * Shift A to the right 5 times to get the block that will hold our sprite
        RORA
        RORA
        RORA
        RORA
        DECA
        ADDA    #VideoRam1_0
        PSHS    A
        STA     MMU_Reg_Bank0_5
        INCA
        STA     MMU_Reg_Bank0_6
        TFR     X,D
        ANDA    #$1F
        ADDD    #$C000+8
        BRA     Red_1
!       CLRB                        * Clear the Carry flag
        ANDA    #$E0
        RORA                        * Shift A to the right 5 times to get the block that will hold our character
        RORA
        RORA
        RORA
        RORA
        ADDA    #VideoRam1_0
        PSHS    A
        STA     MMU_Reg_Bank0_5           * 0 to 3
        INCA
        STA     MMU_Reg_Bank0_6
        TFR     X,D
        ANDA    #$1F
        ADDD    #$A000+8
Red_1:
        TFR     D,X                 * X now points to the top left corner of where the sprite is in RAM ($A000-$DFFF)
        TFR     D,Y                 * Save it in Y also to be used when we draw what was behind the ghost
        PULS    A                   * get the Mem BLK that the sprite is using
        STX     OldRedLocation1     * Save the New Pacman Location over the old
        STA     OldRedMemBlk1       * Save the Mem BLK where Pac Man will be shown
        LDU     RedXPos             * Save the Red X & Y positions
        STU     OldRedXYPos1
        LDB     RedDirection
        STB     OldRedDirection1
        LDU     #RedTableEven
		PULS	CC
		LBCS    RedOddPos1
RedEvenPos1:
        PULS    B                   * Get Sprite Number
        BRA     >
RedOddPos1:
        PULS    B                   * Get Sprite Number
        ADDB    #62                 * 31 entries * 2 bytes each = 62 which points to the Odd sprite table entries
!       STB     OldRedSprite1
        CLRA
        JMP     [D,U]

EraseRedBuff0:
        LDX     OldRedLocation0
        LDD     OldRedMemBlk0       * A=OldRedMemBlk0, B=OldRedSprite0
        STA     MMU_Reg_Bank0_5     * Point to the old Mem Block where Red Ghost was last written
        INCA
        STA     MMU_Reg_Bank0_6
        LDU     #RedTableErase      * Point to the blank version of the Red Ghost sprite
        CLRA
        JSR     [D,U]               * draw the blank Red Ghost sprite

        LDA     OldRedDirection0
        LDU     OldRedLocation0
        LDY     OldRedXYPos0
        JSR     RestoreBehindGhosts


        PULS    B,X                 * restore the registers
        LSLB
        PSHS    B                   * Save pointer to sprite table on the stack
        TFR     X,D
        CMPA    #$09
        BLO     >
        CMPB    #$09
        BLO     >
        CMPA    #$E5
        BHI     >
        CMPB    #$EE
        BHI     >
        BRA     DrawNewRed0
!       PULS    B,PC
DrawNewRed0:
        LDX     #Screen_Start-8-3+128*17  * X points to the top left corner of the playfield (skip over the score and top of the maze)
		LSRA                        * two pixels per byte so divide the pixel location by two
		PSHS	CC					* if the carry is now set then X position is odd
        LEAX    A,X                 * Move to the proper X coordinate (memory location)
        LDA     #128                * 128 bytes per row, move us down in the Y direction
        MUL                         * B = Y coordinate * 128 gives us the memory location to show our Sprite
        LEAX    D,X                 * X now has the place in Video RAM to draw the Sprite
        TFR     X,D                 * Calculate which two memory blocks to get for writing to the screen memory in the Real VideoRAM and store them in $A000-$DFFF
        BITA    #$10
        BNE     >                   * Go if value is not betwwen 0000-0FFF or 2000-2FFF or 4000-4FFF or 6000-6FFF - handle it as normal
        BITA    #$E0
        BEQ     >                   * Skip if it's the range 0000-0FFF as we wont have any in this section
        CLRB                        * otherwise move it a block ahead so the sprite can be erased above it in memory
        ANDA    #$E0
        RORA                        * Shift A to the right 5 times to get the block that will hold our sprite
        RORA
        RORA
        RORA
        RORA
        DECA
        PSHS    A
        STA     MMU_Reg_Bank0_5
        INCA
        STA     MMU_Reg_Bank0_6
        TFR     X,D
        ANDA    #$1F
        ADDD    #$C000+8
        BRA     Red_0
!       CLRB                        * Clear the Carry flag
        ANDA    #$E0
        RORA                        * Shift A to the right 5 times to get the block that will hold our character
        RORA
        RORA
        RORA
        RORA
        PSHS    A
        STA     MMU_Reg_Bank0_5           * 0 to 3
        INCA
        STA     MMU_Reg_Bank0_6
        TFR     X,D
        ANDA    #$1F
        ADDD    #$A000+8
Red_0:
        TFR     D,X                 * X now points to the top left corner of where the sprite is in RAM ($A000-$DFFF)
        TFR     D,Y                 * Save it in Y also to be used when we draw what was behind the ghost
        PULS    A                   * get the Mem BLK that the sprite is using
        STX     OldRedLocation0     * Save the New Pacman Location over the old
        STA     OldRedMemBlk0       * Save the Mem BLK where Pac Man will be shown
        LDU     RedXPos             * Save the Red X & Y positions
        STU     OldRedXYPos0
        LDB     RedDirection
        STB     OldRedDirection0
        LDU     #RedTableEven
		PULS	CC
		LBCS    RedOddPos0
RedEvenPos0:
        PULS    B                   * Get Sprite Number
        BRA     >
RedOddPos0:
        PULS    B                   * Get Sprite Number
        ADDB    #62                 * 31 entries * 2 bytes each = 62 which points to the Odd sprite table entries
!       STB     OldRedSprite0
        CLRA
        JMP     [D,U]

